/*******************************************************************************
* Copyright (C) 2015 Intel Corp. All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice,
*    this list of conditions and the following disclaimer.
*
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this list of conditions and the following disclaimer in the documentation
*    and/or other materials provided with the distribution.
*
*  - Neither the name of Intel Corp. nor the names of its
*    contributors may be used to endorse or promote products derived from this
*    software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL Intel Corp. OR THE CONTRIBUTORS
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using FwUpdateAPI;

namespace FwUpdateCmd
{
    /// <summary>
    /// Class for handling host controller specific commands
    /// </summary>
    internal class ControllerCommandRunner : CommandRunner
    {
        private readonly Dictionary<string, SdkTbtController> _controllers;

        internal ControllerCommandRunner()
        {
            _controllers = LoadControllers();
        }

        protected override SdkTbtBase GetController(string id)
        {
            if (_controllers.ContainsKey(id))
            {
                return _controllers[id];
            }
            throw new TbtException(TbtStatus.SDK_INVALID_CONTROLLER_ID);
        }

        /// <summary>
        /// Loads the controller list from WMI
        /// </summary>
        /// <returns>Controller dictionary (key is controller ID)</returns>
        private static Dictionary<string, SdkTbtController> LoadControllers()
        {
            Dictionary<string, SdkTbtController> controllers;
            try
            {
                controllers = SdkTbtController.GetControllersFromWmi();
            }
            catch
            {
                throw new TbtException(TbtStatus.SDK_LOAD_CONTROLLERS_ERROR);
            }
            if (!controllers.Any())
            {
                throw new TbtException(TbtStatus.SDK_NO_CONTROLLERS,
                                    Resources.NoControllerStringPart0 + "\n"
                                    + Resources.NoDeviceStringPart1 + "\n"
                                    + Resources.NoDeviceStringPart2CMD);
            }
            return controllers;
        }

        /// <summary>
        /// Prints the controller list
        /// </summary>
        internal void EnumControllers()
        {
            foreach (var controller in _controllers)
            {
                Console.WriteLine(controller.Key);
            }
        }
    }
}
